# Snarlygribbly's Snow Machine v2
# Copyright 2012 Royston Harwood

import poser, wx, os, os.path, ConfigParser
import xml.etree.ElementTree as ET
from wx.lib.embeddedimage import PyEmbeddedImage

# Global variables
DEBUG = False
POSER_VERSION = int(float(poser.Version()))
VERSION = '2.2.0'

def ConvertColour(wxcol):
	r, g, b = wxcol
	return (r/255.0, g/255.0, b/255.0)
def GetFloatFromTextCtl(ctl, mult=1.0):
	try:
		value = float(ctl.GetValue()) * mult
	except:
		value = None
	return value
def IsBetween(min, max, value):
	if value >= min and value <= max:
		return True
	else:
		return False
def removeOrphans(mat):
	tree = mat.ShaderTree()
	root = tree.Node(0)
	usedNodes = []
	usedNodes.append(root)
	traverse(root, usedNodes)
	for node in tree.Nodes():
		try:
			index = usedNodes.index(node.InternalName())
		except:
			node.Delete()
	tree.UpdatePreview()
def ShowAlert(message, title=None):
	if os.name == 'posix': return
	if title == None: title = 'The Snow Machine v%s says oops!' % VERSION 
	msg = wx.MessageDialog(None, message, caption=title, style=wx.OK)
	msg.ShowModal()
def traverse(node, usedNodes):
	usedNodes.append(node.InternalName())
	for inp in node.Inputs():
		inNode = inp.InNode()
		if (inNode != None):
			traverse(inNode, usedNodes)

class Main (object):
	def __init__(self):
	
		global POSER_VERSION, VERSION
		
		# Create and configure the UI
		self.ui = UI()
		self.ui.SetTitle(u"Snarlygribbly's Snow Machine v" + VERSION)
		self.ui.SetSize((600, 500))
		if os.name != 'posix':
			self.ui.pgeDistribution.SetBackgroundColour('WHITE')
			self.ui.SetDefaults(None)
		
		# And start the show ...
		self.ui.Show()
		
class mainFrame ( wx.Frame ):
	
	def __init__( self, parent ):
		wx.Frame.__init__ ( self, parent, id = wx.ID_ANY, title = u"Snarlygribbly's Snow Machine v2", pos = wx.DefaultPosition, size = wx.Size( 593,478 ), style = wx.DEFAULT_FRAME_STYLE|wx.TAB_TRAVERSAL )
		
		self.SetSizeHintsSz( wx.DefaultSize, wx.DefaultSize )
		
		bSizer1 = wx.BoxSizer( wx.VERTICAL )
		
		self.Notebook = wx.Notebook( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, 0 )
		self.pgeDistribution = wx.Panel( self.Notebook, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.TAB_TRAVERSAL )
		bSizer3 = wx.BoxSizer( wx.VERTICAL )
		
		bSizer4 = wx.BoxSizer( wx.HORIZONTAL )
		
		self.panelParameters = wx.Panel( self.pgeDistribution, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.DOUBLE_BORDER|wx.TAB_TRAVERSAL )
		bSizer5 = wx.BoxSizer( wx.VERTICAL )
		
		self.m_staticText6 = wx.StaticText( self.panelParameters, wx.ID_ANY, u"PARAMETERS", wx.DefaultPosition, wx.DefaultSize, wx.ALIGN_LEFT )
		self.m_staticText6.Wrap( -1 )
		bSizer5.Add( self.m_staticText6, 0, wx.ALL, 5 )
		
		bSizer9 = wx.BoxSizer( wx.VERTICAL )
		
		bSizerAmount = wx.BoxSizer( wx.HORIZONTAL )
		
		self.m_staticText14 = wx.StaticText( self.panelParameters, wx.ID_ANY, u"Amount:", wx.DefaultPosition, wx.Size( 60,-1 ), wx.ALIGN_RIGHT )
		self.m_staticText14.Wrap( -1 )
		bSizerAmount.Add( self.m_staticText14, 0, wx.ALL, 5 )
		
		self.ctl_Amount = wx.Slider( self.panelParameters, 1001, 50, 10, 90, wx.DefaultPosition, wx.DefaultSize, wx.SL_HORIZONTAL )
		bSizerAmount.Add( self.ctl_Amount, 0, wx.ALL, 5 )
		
		self.txt_Amount = wx.TextCtrl( self.panelParameters, 1002, u"0.50", wx.DefaultPosition, wx.Size( 40,-1 ), wx.TE_PROCESS_ENTER )
		bSizerAmount.Add( self.txt_Amount, 0, wx.ALL, 5 )
		
		
		bSizer9.Add( bSizerAmount, 1, wx.EXPAND, 5 )
		
		bSizer11 = wx.BoxSizer( wx.HORIZONTAL )
		
		self.m_staticText15 = wx.StaticText( self.panelParameters, wx.ID_ANY, u"Depth:", wx.DefaultPosition, wx.Size( 60,-1 ), wx.ALIGN_RIGHT )
		self.m_staticText15.Wrap( -1 )
		bSizer11.Add( self.m_staticText15, 0, wx.ALL, 5 )
		
		self.ctl_Depth = wx.Slider( self.panelParameters, 1003, 50, 0, 200, wx.DefaultPosition, wx.DefaultSize, wx.SL_HORIZONTAL )
		bSizer11.Add( self.ctl_Depth, 0, wx.ALL, 5 )
		
		self.txt_Depth = wx.TextCtrl( self.panelParameters, 1004, u"0.50", wx.DefaultPosition, wx.Size( 40,-1 ), 0 )
		bSizer11.Add( self.txt_Depth, 0, wx.ALL, 5 )
		
		
		bSizer9.Add( bSizer11, 1, wx.EXPAND, 5 )
		
		bSizer12 = wx.BoxSizer( wx.HORIZONTAL )
		
		self.m_staticText16 = wx.StaticText( self.panelParameters, wx.ID_ANY, u"Wiggle:", wx.DefaultPosition, wx.Size( 60,-1 ), wx.ALIGN_RIGHT )
		self.m_staticText16.Wrap( -1 )
		bSizer12.Add( self.m_staticText16, 0, wx.ALL, 5 )
		
		self.ctl_Wiggle = wx.Slider( self.panelParameters, 1005, 3, 1, 8, wx.DefaultPosition, wx.DefaultSize, wx.SL_HORIZONTAL )
		bSizer12.Add( self.ctl_Wiggle, 0, wx.ALL, 5 )
		
		self.txt_Wiggle = wx.TextCtrl( self.panelParameters, wx.ID_ANY, u"3", wx.DefaultPosition, wx.Size( 40,-1 ), 0 )
		bSizer12.Add( self.txt_Wiggle, 0, wx.ALL, 5 )
		
		
		bSizer9.Add( bSizer12, 1, wx.EXPAND, 5 )
		
		bSizer14 = wx.BoxSizer( wx.HORIZONTAL )
		
		self.m_staticText18 = wx.StaticText( self.panelParameters, wx.ID_ANY, u"Bump:", wx.DefaultPosition, wx.Size( 60,-1 ), wx.ALIGN_RIGHT )
		self.m_staticText18.Wrap( -1 )
		bSizer14.Add( self.m_staticText18, 0, wx.ALL, 5 )
		
		self.ctl_Bump = wx.Slider( self.panelParameters, 1009, 2, 0, 2, wx.DefaultPosition, wx.DefaultSize, wx.SL_HORIZONTAL )
		bSizer14.Add( self.ctl_Bump, 0, wx.ALL, 5 )
		
		self.txt_Bump = wx.TextCtrl( self.panelParameters, 1010, u"0.02", wx.DefaultPosition, wx.Size( 40,-1 ), 0 )
		bSizer14.Add( self.txt_Bump, 0, wx.ALL, 5 )
		
		
		bSizer9.Add( bSizer14, 1, wx.EXPAND, 5 )
		
		bSizer19 = wx.BoxSizer( wx.VERTICAL )
		
		self.ctl_Reflect = wx.CheckBox( self.panelParameters, wx.ID_ANY, u"Enable reflection: ", wx.DefaultPosition, wx.DefaultSize, wx.ALIGN_RIGHT )
		self.ctl_Reflect.SetValue(True) 
		bSizer19.Add( self.ctl_Reflect, 0, wx.ALL, 5 )
		
		self.ctl_Smoothing = wx.CheckBox( self.panelParameters, wx.ID_ANY, u"Use smoothing:", wx.DefaultPosition, wx.DefaultSize, wx.ALIGN_RIGHT )
		self.ctl_Smoothing.SetValue(True) 
		bSizer19.Add( self.ctl_Smoothing, 0, wx.ALL, 5 )
		
		self.ctl_SSS = wx.CheckBox( self.panelParameters, wx.ID_ANY, u"Use SSS", wx.DefaultPosition, wx.DefaultSize, wx.ALIGN_RIGHT )
		self.ctl_SSS.SetValue(True) 
		bSizer19.Add( self.ctl_SSS, 0, wx.ALL, 5 )
		
		
		bSizer9.Add( bSizer19, 0, wx.EXPAND, 5 )
		
		
		bSizer5.Add( bSizer9, 0, wx.EXPAND, 5 )
		
		
		self.panelParameters.SetSizer( bSizer5 )
		self.panelParameters.Layout()
		bSizer5.Fit( self.panelParameters )
		bSizer4.Add( self.panelParameters, 1, wx.EXPAND |wx.ALL, 5 )
		
		self.panelDistribution = wx.Panel( self.pgeDistribution, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.DOUBLE_BORDER|wx.TAB_TRAVERSAL )
		bSizer6 = wx.BoxSizer( wx.VERTICAL )
		
		self.m_staticText7 = wx.StaticText( self.panelDistribution, wx.ID_ANY, u"DISTRIBUTION", wx.DefaultPosition, wx.DefaultSize, wx.ALIGN_LEFT )
		self.m_staticText7.Wrap( -1 )
		bSizer6.Add( self.m_staticText7, 0, wx.ALL, 5 )
		
		bSizer14 = wx.BoxSizer( wx.HORIZONTAL )
		
		self.m_staticText9 = wx.StaticText( self.panelDistribution, wx.ID_ANY, u"Current actor:", wx.DefaultPosition, wx.Size( 80,-1 ), 0 )
		self.m_staticText9.Wrap( -1 )
		bSizer14.Add( self.m_staticText9, 0, wx.LEFT|wx.RIGHT|wx.TOP, 5 )
		
		self.txt_CurrentActor = wx.StaticText( self.panelDistribution, wx.ID_ANY, u"No actor selected", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.txt_CurrentActor.Wrap( -1 )
		bSizer14.Add( self.txt_CurrentActor, 0, wx.LEFT|wx.RIGHT|wx.TOP, 5 )
		
		
		bSizer6.Add( bSizer14, 0, wx.EXPAND, 5 )
		
		bSizer15 = wx.BoxSizer( wx.HORIZONTAL )
		
		self.m_staticText11 = wx.StaticText( self.panelDistribution, wx.ID_ANY, u"Actor type:", wx.DefaultPosition, wx.Size( 80,-1 ), 0 )
		self.m_staticText11.Wrap( -1 )
		bSizer15.Add( self.m_staticText11, 0, wx.BOTTOM|wx.LEFT|wx.RIGHT, 5 )
		
		self.txt_ActorType = wx.StaticText( self.panelDistribution, wx.ID_ANY, u"Unknown", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.txt_ActorType.Wrap( -1 )
		bSizer15.Add( self.txt_ActorType, 0, wx.BOTTOM|wx.LEFT, 5 )
		
		
		bSizer6.Add( bSizer15, 0, wx.EXPAND, 5 )
		
		ctl_AddSnowChoices = [ u"Just the current actor", u"Current actor and all other props", u"Everything" ]
		self.ctl_AddSnow = wx.RadioBox( self.panelDistribution, wx.ID_ANY, u"Add snow to:", wx.DefaultPosition, wx.DefaultSize, ctl_AddSnowChoices, 1, wx.RA_SPECIFY_COLS )
		self.ctl_AddSnow.SetSelection( 0 )
		bSizer6.Add( self.ctl_AddSnow, 0, wx.ALL, 5 )
		
		self.m_staticText54 = wx.StaticText( self.panelDistribution, wx.ID_ANY, u"(NB: Hidden actors are not affected)", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_staticText54.Wrap( -1 )
		bSizer6.Add( self.m_staticText54, 0, wx.ALL, 5 )
		
		self.m_staticText141 = wx.StaticText( self.panelDistribution, wx.ID_ANY, u"Apply to:", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_staticText141.Wrap( -1 )
		bSizer6.Add( self.m_staticText141, 0, wx.ALL, 5 )
		
		self.ctl_AllMaterials = wx.CheckBox( self.panelDistribution, wx.ID_ANY, u"All materials", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.ctl_AllMaterials.SetValue(True) 
		bSizer6.Add( self.ctl_AllMaterials, 0, wx.ALL, 5 )
		
		self.ctl_TopOnly = wx.CheckBox( self.panelDistribution, wx.ID_ANY, u"Top surface only", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.ctl_TopOnly.SetValue(True) 
		bSizer6.Add( self.ctl_TopOnly, 0, wx.ALL, 5 )
		
		
		self.panelDistribution.SetSizer( bSizer6 )
		self.panelDistribution.Layout()
		bSizer6.Fit( self.panelDistribution )
		bSizer4.Add( self.panelDistribution, 1, wx.EXPAND |wx.ALL, 5 )
		
		
		bSizer3.Add( bSizer4, 1, wx.EXPAND, 5 )
		
		bSizerHeight = wx.BoxSizer( wx.HORIZONTAL )
		
		self.ctl_LimitByHeight = wx.CheckBox( self.pgeDistribution, wx.ID_ANY, u"Limit by height: ", wx.DefaultPosition, wx.DefaultSize, wx.ALIGN_RIGHT )
		bSizerHeight.Add( self.ctl_LimitByHeight, 0, wx.ALL, 5 )
		
		self.m_staticText8 = wx.StaticText( self.pgeDistribution, wx.ID_ANY, u"Minimum height:", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_staticText8.Wrap( -1 )
		bSizerHeight.Add( self.m_staticText8, 0, wx.ALL, 5 )
		
		self.ctl_Height = wx.Slider( self.pgeDistribution, 1011, 100, 0, 1000, wx.DefaultPosition, wx.DefaultSize, wx.SL_HORIZONTAL )
		bSizerHeight.Add( self.ctl_Height, 0, wx.ALL, 5 )
		
		self.txt_Height = wx.TextCtrl( self.pgeDistribution, 1012, u"100", wx.DefaultPosition, wx.Size( 60,-1 ), 0 )
		bSizerHeight.Add( self.txt_Height, 0, wx.ALL, 5 )
		
		
		bSizer3.Add( bSizerHeight, 0, wx.EXPAND, 5 )
		
		
		self.pgeDistribution.SetSizer( bSizer3 )
		self.pgeDistribution.Layout()
		bSizer3.Fit( self.pgeDistribution )
		self.Notebook.AddPage( self.pgeDistribution, u"Distribution", True )
		self.pgeAdvanced = wx.Panel( self.Notebook, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.TAB_TRAVERSAL )
		bSizer18 = wx.BoxSizer( wx.HORIZONTAL )
		
		bSizer191 = wx.BoxSizer( wx.VERTICAL )
		
		self.m_staticText19 = wx.StaticText( self.pgeAdvanced, wx.ID_ANY, u"Snow settings", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_staticText19.Wrap( -1 )
		self.m_staticText19.SetFont( wx.Font( 8, 74, 90, 92, False, "MS Shell Dlg 2" ) )
		
		bSizer191.Add( self.m_staticText19, 0, wx.ALL, 5 )
		
		bSizer20 = wx.BoxSizer( wx.HORIZONTAL )
		
		self.m_staticText161 = wx.StaticText( self.pgeAdvanced, wx.ID_ANY, u"Snow colour:", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_staticText161.Wrap( -1 )
		bSizer20.Add( self.m_staticText161, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )
		
		self.ctl_Color = wx.ColourPickerCtrl( self.pgeAdvanced, wx.ID_ANY, wx.Colour( 255, 255, 255 ), wx.DefaultPosition, wx.DefaultSize, wx.CLRP_DEFAULT_STYLE|wx.CLRP_SHOW_LABEL|wx.CLRP_USE_TEXTCTRL )
		bSizer20.Add( self.ctl_Color, 0, wx.ALL, 5 )
		
		
		bSizer191.Add( bSizer20, 0, wx.EXPAND, 5 )
		
		self.m_staticline1 = wx.StaticLine( self.pgeAdvanced, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.LI_HORIZONTAL )
		bSizer191.Add( self.m_staticline1, 0, wx.EXPAND |wx.ALL, 5 )
		
		self.m_staticText20 = wx.StaticText( self.pgeAdvanced, wx.ID_ANY, u"Specular settings", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_staticText20.Wrap( -1 )
		self.m_staticText20.SetFont( wx.Font( 8, 74, 90, 92, False, "MS Shell Dlg 2" ) )
		
		bSizer191.Add( self.m_staticText20, 0, wx.ALL, 5 )
		
		bSizer21 = wx.BoxSizer( wx.HORIZONTAL )
		
		self.m_staticText21 = wx.StaticText( self.pgeAdvanced, wx.ID_ANY, u"Spec. Colour", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_staticText21.Wrap( -1 )
		bSizer21.Add( self.m_staticText21, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )
		
		self.ctl_SpecColor = wx.ColourPickerCtrl( self.pgeAdvanced, wx.ID_ANY, wx.Colour( 255, 255, 255 ), wx.DefaultPosition, wx.DefaultSize, wx.CLRP_DEFAULT_STYLE|wx.CLRP_SHOW_LABEL|wx.CLRP_USE_TEXTCTRL )
		bSizer21.Add( self.ctl_SpecColor, 0, wx.ALL, 5 )
		
		
		bSizer191.Add( bSizer21, 0, wx.EXPAND, 5 )
		
		bSizer22 = wx.BoxSizer( wx.HORIZONTAL )
		
		self.m_staticText22 = wx.StaticText( self.pgeAdvanced, wx.ID_ANY, u"Eccentricity", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_staticText22.Wrap( -1 )
		bSizer22.Add( self.m_staticText22, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )
		
		self.ctl_Eccentricity = wx.SpinCtrl( self.pgeAdvanced, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, wx.SP_ARROW_KEYS, 0, 100, 70 )
		bSizer22.Add( self.ctl_Eccentricity, 0, wx.ALL, 5 )
		
		self.m_staticText23 = wx.StaticText( self.pgeAdvanced, wx.ID_ANY, u"%", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_staticText23.Wrap( -1 )
		bSizer22.Add( self.m_staticText23, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )
		
		
		bSizer191.Add( bSizer22, 0, wx.EXPAND, 5 )
		
		bSizer23 = wx.BoxSizer( wx.HORIZONTAL )
		
		self.m_staticText24 = wx.StaticText( self.pgeAdvanced, wx.ID_ANY, u"        RollOff", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_staticText24.Wrap( -1 )
		bSizer23.Add( self.m_staticText24, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )
		
		self.ctl_RollOff = wx.SpinCtrl( self.pgeAdvanced, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, wx.SP_ARROW_KEYS, 0, 100, 30 )
		bSizer23.Add( self.ctl_RollOff, 0, wx.ALL, 5 )
		
		self.m_staticText25 = wx.StaticText( self.pgeAdvanced, wx.ID_ANY, u"%", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_staticText25.Wrap( -1 )
		bSizer23.Add( self.m_staticText25, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )
		
		
		bSizer191.Add( bSizer23, 0, wx.EXPAND, 5 )
		
		bSizer24 = wx.BoxSizer( wx.HORIZONTAL )
		
		self.m_staticText26 = wx.StaticText( self.pgeAdvanced, wx.ID_ANY, u" Reflectivity", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_staticText26.Wrap( -1 )
		bSizer24.Add( self.m_staticText26, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )
		
		self.ctl_Reflectivity = wx.SpinCtrl( self.pgeAdvanced, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, wx.SP_ARROW_KEYS, 0, 1000, 100 )
		bSizer24.Add( self.ctl_Reflectivity, 0, wx.ALL, 5 )
		
		self.m_staticText27 = wx.StaticText( self.pgeAdvanced, wx.ID_ANY, u"%", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_staticText27.Wrap( -1 )
		bSizer24.Add( self.m_staticText27, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )
		
		
		bSizer191.Add( bSizer24, 0, wx.EXPAND, 5 )
		
		bSizer29 = wx.BoxSizer( wx.HORIZONTAL )
		
		self.ctl_Snowy = wx.Button( self.pgeAdvanced, wx.ID_ANY, u"Snowy", wx.DefaultPosition, wx.DefaultSize, 0 )
		bSizer29.Add( self.ctl_Snowy, 0, wx.ALL, 5 )
		
		self.ctl_Icy = wx.Button( self.pgeAdvanced, wx.ID_ANY, u"Icy", wx.DefaultPosition, wx.DefaultSize, 0 )
		bSizer29.Add( self.ctl_Icy, 0, wx.ALL, 5 )
		
		
		bSizer191.Add( bSizer29, 0, wx.EXPAND, 5 )
		
		self.m_staticline2 = wx.StaticLine( self.pgeAdvanced, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.LI_HORIZONTAL )
		bSizer191.Add( self.m_staticline2, 0, wx.EXPAND |wx.ALL, 5 )
		
		
		bSizer18.Add( bSizer191, 1, wx.EXPAND, 5 )
		
		bSizer21 = wx.BoxSizer( wx.VERTICAL )
		
		self.m_staticText28 = wx.StaticText( self.pgeAdvanced, wx.ID_ANY, u"Reflection settings", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_staticText28.Wrap( -1 )
		self.m_staticText28.SetFont( wx.Font( 8, 74, 90, 92, False, "MS Shell Dlg 2" ) )
		
		bSizer21.Add( self.m_staticText28, 0, wx.ALL, 5 )
		
		bSizer25 = wx.BoxSizer( wx.HORIZONTAL )
		
		self.m_staticText29 = wx.StaticText( self.pgeAdvanced, wx.ID_ANY, u"Quality", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_staticText29.Wrap( -1 )
		self.m_staticText29.SetMinSize( wx.Size( 60,-1 ) )
		
		bSizer25.Add( self.m_staticText29, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )
		
		self.ctl_ReflectQuality = wx.SpinCtrl( self.pgeAdvanced, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, wx.SP_ARROW_KEYS, 0, 100, 20 )
		bSizer25.Add( self.ctl_ReflectQuality, 0, wx.ALL, 5 )
		
		self.m_staticText30 = wx.StaticText( self.pgeAdvanced, wx.ID_ANY, u"%", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_staticText30.Wrap( -1 )
		bSizer25.Add( self.m_staticText30, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )
		
		
		bSizer21.Add( bSizer25, 0, wx.EXPAND, 5 )
		
		bSizer26 = wx.BoxSizer( wx.HORIZONTAL )
		
		self.m_staticText31 = wx.StaticText( self.pgeAdvanced, wx.ID_ANY, u"Softness", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_staticText31.Wrap( -1 )
		self.m_staticText31.SetMinSize( wx.Size( 60,-1 ) )
		
		bSizer26.Add( self.m_staticText31, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )
		
		self.ctl_ReflectSoftness = wx.SpinCtrl( self.pgeAdvanced, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, wx.SP_ARROW_KEYS, 0, 400, 0 )
		bSizer26.Add( self.ctl_ReflectSoftness, 0, wx.ALL, 5 )
		
		self.m_staticText32 = wx.StaticText( self.pgeAdvanced, wx.ID_ANY, u"%", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_staticText32.Wrap( -1 )
		bSizer26.Add( self.m_staticText32, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )
		
		
		bSizer21.Add( bSizer26, 0, wx.EXPAND, 5 )
		
		bSizer30 = wx.BoxSizer( wx.HORIZONTAL )
		
		self.m_staticText36 = wx.StaticText( self.pgeAdvanced, wx.ID_ANY, u"Reflectivity", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_staticText36.Wrap( -1 )
		self.m_staticText36.SetMinSize( wx.Size( 60,-1 ) )
		
		bSizer30.Add( self.m_staticText36, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )
		
		self.ctl_ReflectAmount = wx.SpinCtrl( self.pgeAdvanced, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, wx.SP_ARROW_KEYS, 10, 300, 14 )
		bSizer30.Add( self.ctl_ReflectAmount, 0, wx.ALL, 5 )
		
		self.m_staticText37 = wx.StaticText( self.pgeAdvanced, wx.ID_ANY, u"(10 to 30)", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_staticText37.Wrap( -1 )
		bSizer30.Add( self.m_staticText37, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )
		
		
		bSizer21.Add( bSizer30, 0, wx.EXPAND, 5 )
		
		self.m_staticText33 = wx.StaticText( self.pgeAdvanced, wx.ID_ANY, u"Colours for fake environmental reflections", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_staticText33.Wrap( -1 )
		bSizer21.Add( self.m_staticText33, 0, wx.ALL, 5 )
		
		bSizer27 = wx.BoxSizer( wx.HORIZONTAL )
		
		self.m_staticText34 = wx.StaticText( self.pgeAdvanced, wx.ID_ANY, u"Colour 1", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_staticText34.Wrap( -1 )
		bSizer27.Add( self.m_staticText34, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )
		
		self.ctl_Cloud1 = wx.ColourPickerCtrl( self.pgeAdvanced, wx.ID_ANY, wx.Colour( 204, 204, 255 ), wx.DefaultPosition, wx.DefaultSize, wx.CLRP_DEFAULT_STYLE|wx.CLRP_SHOW_LABEL|wx.CLRP_USE_TEXTCTRL )
		bSizer27.Add( self.ctl_Cloud1, 0, wx.ALL, 5 )
		
		
		bSizer21.Add( bSizer27, 0, wx.EXPAND, 5 )
		
		bSizer28 = wx.BoxSizer( wx.HORIZONTAL )
		
		self.m_staticText35 = wx.StaticText( self.pgeAdvanced, wx.ID_ANY, u"Colour 2", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_staticText35.Wrap( -1 )
		bSizer28.Add( self.m_staticText35, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )
		
		self.ctl_Cloud2 = wx.ColourPickerCtrl( self.pgeAdvanced, wx.ID_ANY, wx.Colour( 255, 255, 255 ), wx.DefaultPosition, wx.DefaultSize, wx.CLRP_DEFAULT_STYLE|wx.CLRP_SHOW_LABEL|wx.CLRP_USE_TEXTCTRL )
		bSizer28.Add( self.ctl_Cloud2, 0, wx.ALL, 5 )
		
		
		bSizer21.Add( bSizer28, 0, wx.EXPAND, 5 )
		
		self.m_staticline5 = wx.StaticLine( self.pgeAdvanced, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.LI_HORIZONTAL )
		bSizer21.Add( self.m_staticline5, 0, wx.EXPAND |wx.ALL, 5 )
		
		self.m_staticText57 = wx.StaticText( self.pgeAdvanced, wx.ID_ANY, u"Opacity", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_staticText57.Wrap( -1 )
		self.m_staticText57.SetFont( wx.Font( 8, 74, 90, 92, False, "MS Shell Dlg 2" ) )
		
		bSizer21.Add( self.m_staticText57, 0, wx.ALL, 5 )
		
		bSizer31 = wx.BoxSizer( wx.HORIZONTAL )
		
		self.txt_Opacity = wx.StaticText( self.pgeAdvanced, wx.ID_ANY, u"Opacity", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.txt_Opacity.Wrap( -1 )
		self.txt_Opacity.SetMinSize( wx.Size( 60,-1 ) )
		
		bSizer31.Add( self.txt_Opacity, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )
		
		self.ctl_Opacity = wx.SpinCtrl( self.pgeAdvanced, wx.ID_ANY, u"100", wx.DefaultPosition, wx.DefaultSize, wx.SP_ARROW_KEYS, 0, 100, 100 )
		bSizer31.Add( self.ctl_Opacity, 0, wx.ALL, 5 )
		
		self.m_staticText55 = wx.StaticText( self.pgeAdvanced, wx.ID_ANY, u"%", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_staticText55.Wrap( -1 )
		bSizer31.Add( self.m_staticText55, 0, wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5 )
		
		
		bSizer21.Add( bSizer31, 0, wx.EXPAND, 5 )
		
		
		bSizer18.Add( bSizer21, 1, wx.EXPAND, 5 )
		
		
		self.pgeAdvanced.SetSizer( bSizer18 )
		self.pgeAdvanced.Layout()
		bSizer18.Fit( self.pgeAdvanced )
		self.Notebook.AddPage( self.pgeAdvanced, u"Advanced", False )
		self.pgeCredits = wx.Panel( self.Notebook, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.TAB_TRAVERSAL )
		bSizer17 = wx.BoxSizer( wx.VERTICAL )
		
		self.m_staticText142 = wx.StaticText( self.pgeCredits, wx.ID_ANY, u"Programming", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_staticText142.Wrap( -1 )
		self.m_staticText142.SetFont( wx.Font( 8, 74, 90, 92, False, "MS Shell Dlg 2" ) )
		
		bSizer17.Add( self.m_staticText142, 0, wx.ALIGN_CENTER_HORIZONTAL|wx.ALL, 5 )
		
		self.m_staticText151 = wx.StaticText( self.pgeCredits, wx.ID_ANY, u"Snarlygribbly", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_staticText151.Wrap( -1 )
		bSizer17.Add( self.m_staticText151, 0, wx.ALIGN_CENTER_HORIZONTAL, 5 )
		
		self.m_staticText162 = wx.StaticText( self.pgeCredits, wx.ID_ANY, u"User guide", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_staticText162.Wrap( -1 )
		self.m_staticText162.SetFont( wx.Font( 8, 74, 90, 92, False, "MS Shell Dlg 2" ) )
		
		bSizer17.Add( self.m_staticText162, 0, wx.ALIGN_CENTER_HORIZONTAL|wx.ALL, 5 )
		
		self.m_staticText17 = wx.StaticText( self.pgeCredits, wx.ID_ANY, u"basicwiz", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_staticText17.Wrap( -1 )
		bSizer17.Add( self.m_staticText17, 0, wx.ALIGN_CENTER_HORIZONTAL, 5 )
		
		self.m_staticText181 = wx.StaticText( self.pgeCredits, wx.ID_ANY, u"Testers", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_staticText181.Wrap( -1 )
		self.m_staticText181.SetFont( wx.Font( 8, 74, 90, 92, False, "MS Shell Dlg 2" ) )
		
		bSizer17.Add( self.m_staticText181, 0, wx.ALIGN_CENTER_HORIZONTAL|wx.ALL, 5 )
		
		self.m_staticText38 = wx.StaticText( self.pgeCredits, wx.ID_ANY, u"basicwiz", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_staticText38.Wrap( -1 )
		bSizer17.Add( self.m_staticText38, 0, wx.ALIGN_CENTER_HORIZONTAL, 5 )
		
		self.m_staticText48 = wx.StaticText( self.pgeCredits, wx.ID_ANY, u"Caisson", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_staticText48.Wrap( -1 )
		bSizer17.Add( self.m_staticText48, 0, wx.ALIGN_CENTER_HORIZONTAL, 5 )
		
		self.m_staticText49 = wx.StaticText( self.pgeCredits, wx.ID_ANY, u"Christel", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_staticText49.Wrap( -1 )
		bSizer17.Add( self.m_staticText49, 0, wx.ALIGN_CENTER_HORIZONTAL, 5 )
		
		self.m_staticText50 = wx.StaticText( self.pgeCredits, wx.ID_ANY, u"ericwalters", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_staticText50.Wrap( -1 )
		bSizer17.Add( self.m_staticText50, 0, wx.ALIGN_CENTER_HORIZONTAL, 5 )
		
		self.m_staticText51 = wx.StaticText( self.pgeCredits, wx.ID_ANY, u"Featherwitch", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_staticText51.Wrap( -1 )
		bSizer17.Add( self.m_staticText51, 0, wx.ALIGN_CENTER_HORIZONTAL, 5 )
		
		self.m_staticText39 = wx.StaticText( self.pgeCredits, wx.ID_ANY, u"Gareee", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_staticText39.Wrap( -1 )
		bSizer17.Add( self.m_staticText39, 0, wx.ALIGN_CENTER_HORIZONTAL, 5 )
		
		self.m_staticText46 = wx.StaticText( self.pgeCredits, wx.ID_ANY, u"GeneralNutt", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_staticText46.Wrap( -1 )
		bSizer17.Add( self.m_staticText46, 0, wx.ALIGN_CENTER_HORIZONTAL, 5 )
		
		self.m_staticText44 = wx.StaticText( self.pgeCredits, wx.ID_ANY, u"Glitterati3D", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_staticText44.Wrap( -1 )
		bSizer17.Add( self.m_staticText44, 0, wx.ALIGN_CENTER_HORIZONTAL, 5 )
		
		self.m_staticText53 = wx.StaticText( self.pgeCredits, wx.ID_ANY, u"KageRyu", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_staticText53.Wrap( -1 )
		bSizer17.Add( self.m_staticText53, 0, wx.ALIGN_CENTER_HORIZONTAL, 5 )
		
		self.m_staticText40 = wx.StaticText( self.pgeCredits, wx.ID_ANY, u"operaguy", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_staticText40.Wrap( -1 )
		bSizer17.Add( self.m_staticText40, 0, wx.ALIGN_CENTER_HORIZONTAL, 5 )
		
		self.m_staticText45 = wx.StaticText( self.pgeCredits, wx.ID_ANY, u"perlk", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_staticText45.Wrap( -1 )
		bSizer17.Add( self.m_staticText45, 0, wx.ALIGN_CENTER_HORIZONTAL, 5 )
		
		self.m_staticText52 = wx.StaticText( self.pgeCredits, wx.ID_ANY, u"foxylady", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_staticText52.Wrap( -1 )
		bSizer17.Add( self.m_staticText52, 0, wx.ALIGN_CENTER_HORIZONTAL, 5 )
		
		self.m_staticText42 = wx.StaticText( self.pgeCredits, wx.ID_ANY, u"Robert Luis Rabello", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_staticText42.Wrap( -1 )
		bSizer17.Add( self.m_staticText42, 0, wx.ALIGN_CENTER_HORIZONTAL, 5 )
		
		self.m_staticText47 = wx.StaticText( self.pgeCredits, wx.ID_ANY, u"Tool_Pepe", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_staticText47.Wrap( -1 )
		bSizer17.Add( self.m_staticText47, 0, wx.ALIGN_CENTER_HORIZONTAL, 5 )
		
		self.m_staticText41 = wx.StaticText( self.pgeCredits, wx.ID_ANY, u"Wimdvb", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_staticText41.Wrap( -1 )
		bSizer17.Add( self.m_staticText41, 0, wx.ALIGN_CENTER_HORIZONTAL, 5 )
		
		
		self.pgeCredits.SetSizer( bSizer17 )
		self.pgeCredits.Layout()
		bSizer17.Fit( self.pgeCredits )
		self.Notebook.AddPage( self.pgeCredits, u"Credits", False )
		
		bSizer1.Add( self.Notebook, 1, wx.EXPAND |wx.ALL, 5 )
		
		bSizerButtons = wx.BoxSizer( wx.HORIZONTAL )
		
		self.btnMakeSnow = wx.Button( self, wx.ID_ANY, u"Make snow!", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.btnMakeSnow.SetBackgroundColour( wx.SystemSettings.GetColour( wx.SYS_COLOUR_3DLIGHT ) )
		
		bSizerButtons.Add( self.btnMakeSnow, 0, wx.ALL, 5 )
		
		self.btnDefaults = wx.Button( self, wx.ID_ANY, u"Defaults", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.btnDefaults.SetBackgroundColour( wx.SystemSettings.GetColour( wx.SYS_COLOUR_3DLIGHT ) )
		
		bSizerButtons.Add( self.btnDefaults, 0, wx.ALL, 5 )
		
		self.btnSavePreset = wx.Button( self, wx.ID_ANY, u"Save as preset", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.btnSavePreset.SetBackgroundColour( wx.SystemSettings.GetColour( wx.SYS_COLOUR_3DLIGHT ) )
		
		bSizerButtons.Add( self.btnSavePreset, 0, wx.ALL, 5 )
		
		self.btnGet = wx.Button( self, wx.ID_ANY, u"Get", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.btnGet.SetBackgroundColour( wx.SystemSettings.GetColour( wx.SYS_COLOUR_3DLIGHT ) )
		
		bSizerButtons.Add( self.btnGet, 0, wx.ALL, 5 )
		
		self.btnExit = wx.Button( self, wx.ID_ANY, u"Exit", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.btnExit.SetBackgroundColour( wx.SystemSettings.GetColour( wx.SYS_COLOUR_3DLIGHT ) )
		
		bSizerButtons.Add( self.btnExit, 0, wx.ALL, 5 )
		
		
		bSizer1.Add( bSizerButtons, 0, wx.EXPAND, 5 )
		
		
		self.SetSizer( bSizer1 )
		self.Layout()
		self.MenuBar = wx.MenuBar( 0 )
		self.menuFile = wx.Menu()
		self.itemLoad = wx.MenuItem( self.menuFile, wx.ID_ANY, u"Load from XML", wx.EmptyString, wx.ITEM_NORMAL )
		self.menuFile.AppendItem( self.itemLoad )
		
		self.itemSave = wx.MenuItem( self.menuFile, wx.ID_ANY, u"Save to XML", wx.EmptyString, wx.ITEM_NORMAL )
		self.menuFile.AppendItem( self.itemSave )
		
		self.menuFile.AppendSeparator()
		
		self.itemQuit = wx.MenuItem( self.menuFile, wx.ID_ANY, u"Quit", wx.EmptyString, wx.ITEM_NORMAL )
		self.menuFile.AppendItem( self.itemQuit )
		
		self.MenuBar.Append( self.menuFile, u"File" ) 
		
		self.menuAction = wx.Menu()
		self.itemMakeSnow = wx.MenuItem( self.menuAction, wx.ID_ANY, u"Make snow!", wx.EmptyString, wx.ITEM_NORMAL )
		self.menuAction.AppendItem( self.itemMakeSnow )
		
		self.itemDefaults = wx.MenuItem( self.menuAction, wx.ID_ANY, u"Reset to defaults", wx.EmptyString, wx.ITEM_NORMAL )
		self.menuAction.AppendItem( self.itemDefaults )
		
		self.itemGet = wx.MenuItem( self.menuAction, wx.ID_ANY, u"Get values from scene", wx.EmptyString, wx.ITEM_NORMAL )
		self.menuAction.AppendItem( self.itemGet )
		
		self.MenuBar.Append( self.menuAction, u"Action" ) 
		
		self.mnu_Presets = wx.Menu()
		self.itm_SavePreset = wx.MenuItem( self.mnu_Presets, wx.ID_ANY, u"Save preset ...", wx.EmptyString, wx.ITEM_NORMAL )
		self.mnu_Presets.AppendItem( self.itm_SavePreset )
		
		self.itm_ManagePresets = wx.MenuItem( self.mnu_Presets, wx.ID_ANY, u"Manage presets", wx.EmptyString, wx.ITEM_NORMAL )
		self.mnu_Presets.AppendItem( self.itm_ManagePresets )
		
		self.mnu_Presets.AppendSeparator()
		
		self.itm_ImportPreset = wx.MenuItem( self.mnu_Presets, wx.ID_ANY, u"Import", wx.EmptyString, wx.ITEM_NORMAL )
		self.mnu_Presets.AppendItem( self.itm_ImportPreset )
		
		self.itm_ExportPreset = wx.MenuItem( self.mnu_Presets, wx.ID_ANY, u"Export", wx.EmptyString, wx.ITEM_NORMAL )
		self.mnu_Presets.AppendItem( self.itm_ExportPreset )
		
		self.mnu_Presets.AppendSeparator()
		
		self.MenuBar.Append( self.mnu_Presets, u"Presets" ) 
		
		self.SetMenuBar( self.MenuBar )
		
		self.ctl_StatusBar = self.CreateStatusBar( 1, wx.ST_SIZEGRIP, wx.ID_ANY )
		
		self.Centre( wx.BOTH )
		
		# Connect Events
		self.ctl_Amount.Bind( wx.EVT_SCROLL, self.UpdateAmount )
		self.txt_Amount.Bind( wx.EVT_TEXT, self.UpdateAmount )
		self.txt_Amount.Bind( wx.EVT_TEXT_ENTER, self.UpdateAmount )
		self.ctl_Depth.Bind( wx.EVT_SCROLL, self.UpdateDepth )
		self.txt_Depth.Bind( wx.EVT_TEXT, self.UpdateDepth )
		self.txt_Depth.Bind( wx.EVT_TEXT_ENTER, self.UpdateDepth )
		self.ctl_Wiggle.Bind( wx.EVT_SCROLL, self.UpdateWiggle )
		self.txt_Wiggle.Bind( wx.EVT_TEXT, self.UpdateWiggle )
		self.txt_Wiggle.Bind( wx.EVT_TEXT_ENTER, self.UpdateWiggle )
		self.ctl_Bump.Bind( wx.EVT_SCROLL, self.UpdateBump )
		self.txt_Bump.Bind( wx.EVT_TEXT, self.UpdateBump )
		self.txt_Bump.Bind( wx.EVT_TEXT_ENTER, self.UpdateBump )
		self.ctl_AllMaterials.Bind( wx.EVT_CHECKBOX, self.AllMats )
		self.ctl_TopOnly.Bind( wx.EVT_CHECKBOX, self.TopOnlyChanged )
		self.ctl_Height.Bind( wx.EVT_SCROLL, self.UpdateHeight )
		self.txt_Height.Bind( wx.EVT_TEXT, self.UpdateHeight )
		self.txt_Height.Bind( wx.EVT_TEXT_ENTER, self.UpdateHeight )
		self.ctl_Snowy.Bind( wx.EVT_BUTTON, self.Snowy )
		self.ctl_Icy.Bind( wx.EVT_BUTTON, self.Icy )
		self.btnMakeSnow.Bind( wx.EVT_BUTTON, self.MakeSnow )
		self.btnDefaults.Bind( wx.EVT_BUTTON, self.SetDefaults )
		self.btnSavePreset.Bind( wx.EVT_BUTTON, self.SavePreset )
		self.btnGet.Bind( wx.EVT_BUTTON, self.GetFromScene )
		self.btnExit.Bind( wx.EVT_BUTTON, self.Quit )
		self.Bind( wx.EVT_MENU, self.MenuLoad, id = self.itemLoad.GetId() )
		self.Bind( wx.EVT_MENU, self.MenuSave, id = self.itemSave.GetId() )
		self.Bind( wx.EVT_MENU, self.Quit, id = self.itemQuit.GetId() )
		self.Bind( wx.EVT_MENU, self.MakeSnow, id = self.itemMakeSnow.GetId() )
		self.Bind( wx.EVT_MENU, self.SetDefaults, id = self.itemDefaults.GetId() )
		self.Bind( wx.EVT_MENU, self.GetFromScene, id = self.itemGet.GetId() )
		self.Bind( wx.EVT_MENU, self.SavePreset, id = self.itm_SavePreset.GetId() )
		self.Bind( wx.EVT_MENU, self.ManagePreset, id = self.itm_ManagePresets.GetId() )
		self.Bind( wx.EVT_MENU, self.PresetImport, id = self.itm_ImportPreset.GetId() )
		self.Bind( wx.EVT_MENU, self.PresetExport, id = self.itm_ExportPreset.GetId() )
	
	def __del__( self ):
		pass
	
	
	# Virtual event handlers, overide them in your derived class
	def UpdateAmount( self, event ):
		event.Skip()
	
	
	
	def UpdateDepth( self, event ):
		event.Skip()
	
	
	
	def UpdateWiggle( self, event ):
		event.Skip()
	
	
	
	def UpdateBump( self, event ):
		event.Skip()
	
	
	
	def AllMats( self, event ):
		event.Skip()
	
	def TopOnlyChanged( self, event ):
		event.Skip()
	
	def UpdateHeight( self, event ):
		event.Skip()
	
	
	
	def Snowy( self, event ):
		event.Skip()
	
	def Icy( self, event ):
		event.Skip()
	
	def MakeSnow( self, event ):
		event.Skip()
	
	def SetDefaults( self, event ):
		event.Skip()
	
	def SavePreset( self, event ):
		event.Skip()
	
	def GetFromScene( self, event ):
		event.Skip()
	
	def Quit( self, event ):
		event.Skip()
	
	def MenuLoad( self, event ):
		event.Skip()
	
	def MenuSave( self, event ):
		event.Skip()
	
	
	
	
	
	
	def ManagePreset( self, event ):
		event.Skip()
	
	def PresetImport( self, event ):
		event.Skip()
	
	def PresetExport( self, event ):
		event.Skip()
	

	
class PresetEditor ( wx.Dialog ):
	
	def __init__( self, parent ):
		wx.Dialog.__init__ ( self, parent, id = wx.ID_ANY, title = u"Preset editor", pos = wx.DefaultPosition, size = wx.Size( 257,432 ), style = wx.CAPTION|wx.DEFAULT_DIALOG_STYLE )
		
		self.SetSizeHintsSz( wx.DefaultSize, wx.DefaultSize )
		
		box_Configure = wx.BoxSizer( wx.VERTICAL )
		
		ctl_PresetListChoices = []
		self.ctl_PresetList = wx.CheckListBox( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, ctl_PresetListChoices, wx.LB_ALWAYS_SB|wx.LB_MULTIPLE|wx.LB_SORT )
		box_Configure.Add( self.ctl_PresetList, 1, wx.ALL|wx.EXPAND, 5 )
		
		box_Select = wx.BoxSizer( wx.HORIZONTAL )
		
		self.ctl_SelectAll = wx.Button( self, wx.ID_ANY, u"Select All", wx.DefaultPosition, wx.DefaultSize, 0 )
		box_Select.Add( self.ctl_SelectAll, 0, wx.ALL, 5 )
		
		self.ctl_SelectNone = wx.Button( self, wx.ID_ANY, u"Select None", wx.DefaultPosition, wx.DefaultSize, 0 )
		box_Select.Add( self.ctl_SelectNone, 0, wx.ALL, 5 )
		
		
		box_Configure.Add( box_Select, 0, wx.EXPAND, 5 )
		
		box_Buttons = wx.BoxSizer( wx.HORIZONTAL )
		
		self.ctl_SelectPresets = wx.Button( self, wx.ID_OK, u"Select", wx.DefaultPosition, wx.DefaultSize, 0 )
		box_Buttons.Add( self.ctl_SelectPresets, 0, wx.ALL, 5 )
		
		self.ctl_Cancel = wx.Button( self, wx.ID_CANCEL, u"Cancel", wx.DefaultPosition, wx.DefaultSize, 0 )
		box_Buttons.Add( self.ctl_Cancel, 0, wx.ALL, 5 )
		
		
		box_Configure.Add( box_Buttons, 0, wx.EXPAND, 5 )
		
		
		self.SetSizer( box_Configure )
		self.Layout()
		
		self.Centre( wx.BOTH )
		
		# Connect Events
		self.ctl_SelectAll.Bind( wx.EVT_BUTTON, self.SelectAll )
		self.ctl_SelectNone.Bind( wx.EVT_BUTTON, self.SelectNone )
		self.ctl_SelectPresets.Bind( wx.EVT_BUTTON, self.SelectPresets )
		self.ctl_Cancel.Bind( wx.EVT_BUTTON, self.Cancel )
	
	def __del__( self ):
		# Disconnect Events
		self.ctl_SelectAll.Unbind( wx.EVT_BUTTON, None )
		self.ctl_SelectNone.Unbind( wx.EVT_BUTTON, None )
		self.ctl_SelectPresets.Unbind( wx.EVT_BUTTON, None )
		self.ctl_Cancel.Unbind( wx.EVT_BUTTON, None )
	
	
	# Virtual event handlers, overide them in your derived class
	def SelectAll( self, event ):
		event.Skip()
	
	def SelectNone( self, event ):
		event.Skip()
	
	def SelectPresets( self, event ):
		event.Skip()
	
	def Cancel( self, event ):
		event.Skip()

class Mat (object):
	def __init__(self, material):
		
		self.material = material
		self.root = material.ShaderTree().Node(0)
		self.tree = material.ShaderTree()
		
		self.codes = {
			"anisotropic" : poser.kNodeTypeCodeANISOTROPIC,
			"blender" : poser.kNodeTypeCodeBLENDER,
			"blinn" : poser.kNodeTypeCodeBLINN,
			"clouds" : poser.kNodeTypeCodeCLOUDS,
			"color" : poser.kNodeTypeCodeSIMPLECOLOR,
			"color_math" : poser.kNodeTypeCodeCOLORMATH,
			"component" : poser.kNodeTypeCodeCOMP,
			"custom_scatter" : "custom_scatter",
			"diffuse" : poser.kNodeTypeCodeDIFFUSE,
			"fractal" : poser.kNodeTypeCodeFRACTALSUM,
			"fresnel_blend" : "fresnel_blend",
			"hsv" : poser.kNodeTypeCodeHSV,
			"imagemap" : poser.kNodeTypeCodeIMAGEMAP,
			"math" : poser.kNodeTypeCodeMATH,
			"microfacet" : "ks_microfacet",
			"n" : poser.kNodeTypeCodeN,
			"noise" : poser.kNodeTypeCodeNOISE,
			"p" : poser.kNodeTypeCodeP,
			"reflect" : poser.kNodeTypeCodeREFLECT,
			"scatter" : poser.kNodeTypeCodeSCATTER,
			'specular' : poser.kNodeTypeCodeSPECULAR,
			'sphere_map' : poser.kNodeTypeCodeSPHEREMAP,
			"turbulence" : poser.kNodeTypeCodeTURBULENCE,
			}
			
	def AddNode(self, type, name=None, loc=None, collapsed=True, **kw):
		node = self.tree.CreateNode(self.codes[type])
		if loc: node.SetLocation(loc[0], loc[1])
		if name: node.SetName(name)
		node.SetInputsCollapsed(collapsed)
		for key, value in kw.items():
			input = node.InputByInternalName(key)
			self.SetInputValue(input, value)
		return node

	def ConfigureNode(self, node, loc=None, name=None, showPreview=None, showInputs=None):
		
		if loc: node.SetLocation(loc[0], loc[1])
		if name: node.SetName(name)
		if showPreview != None: node.SetPreviewCollapsed((0, 1)[showPreview == True])
		if showInputs != None: node.SetInputsCollapsed((1, 0)[showInputs == True])
	
	def ConnectNodes(self, n1, n2, inp):
		n1.ConnectToInput(n2.InputByInternalName(inp))
	
	def ConnectToPS(self, node, input, value=None):
		i = self.root.InputByInternalName(input)
		node.ConnectToInput(i)
		if value:
			self.SetInputValue(i, value)
	
	def GetPSNode(self, input):
		return self.root.InputByInternalName(input).InNode()
		
	def GetPSValue(self, input):
		return self.root.InputByInternalName(input).Value()

	def RemovePSNode(self, input):
		self.root.InputByInternalName(input).Disconnect()
	
	def SetInputValue(self, input, value):
		if input:
			inputType = input.Type()
			if inputType in [0, 4, 6]:
				try: input.SetFloat(value)
				except: pass
			elif inputType in [1, 2]:
				try: 
					r, g, b = value
					input.SetColor(r, g, b)
				except: pass
	
	def SetPSValue(self, input, value):
		self.SetInputValue(self.root.InputByInternalName(input), value)
		
class SnowMachine (object):
	def	__init__(self, ui):
		
		self.ui = ui

		# Get details of current actor
		self.ActorData()
		
	def ActorData(self):
		self.actor = poser.Scene().CurrentActor()
		self.name, self.type = self.GetActorData(self.actor)

	def AddBumpNodes(self, mat, bump, offset=None):
	
		if offset:
			x, y = offset
		else:
			x = y = 0
		
		bump_amt = mat.GetPSValue('Displacement')
		
		b01 = mat.AddNode('math', loc=(0+x, 0+y), collapsed=False, Math_Argument=1.0, Value_1=bump_amt, Value_2=0.0, name='Bump amount')
		b02 = mat.AddNode('math', loc=(0+x, 80+y), collapsed=True, Math_Argument=4.0, Value_1=1.0, Value_2=1.0, name='b02')
		b03 = mat.AddNode('math', loc=(0+x, 110+y), collapsed=True, Math_Argument=3.0, Value_1=1.0, Value_2=1.0, name='b03')
		b04 = mat.AddNode('noise', loc=(0+x, 140+y), collapsed=True, name='b04')
		
		mat.ConnectNodes(b01, b02, 'Value_2')
		mat.ConnectNodes(b02, b03, 'Value_1')
		mat.ConnectNodes(b04, b03, 'Value_2')
		mat.ConnectNodes(bump, b02, 'Value_1')
		
		return b03
		
	def AddDisplacementNodes(self, mat, disp, offset=None):
	
		if offset:
			x, y = offset
		else:
			x = y = 0
			
		disp_amt = mat.GetPSValue('Displacement')	
			
		d01 = mat.AddNode('math', loc=(0+x, 0+y), collapsed=True, Math_Argument=1.0, Value_1=disp_amt, Value_2=0.0, name='Displacement')
		d02 = mat.AddNode('math', loc=(0+x, 30+y), collapsed=True, Math_Argument=4.0, Value_1=1.0, Value_2=1.0, name='d02')
		d03 = mat.AddNode('math', loc=(0+x, 60+y), collapsed=False, Math_Argument=3.0, Value_1=1.0, Value_2=1.0, name='d03')
		
		mat.ConnectNodes(d01, d02, 'Value_2')
		mat.ConnectNodes(d02, d03, 'Value_2')
		mat.ConnectNodes(disp, d03, 'Value_1')
		
		return d03
		
	def AddHeightNodes(self, mat, min, offset=None):

		if offset:
			x, y = offset
		else:
			x = y = 0
			
		h01 = mat.AddNode('math', loc=(0+x, 0+y), Math_Argument=17, Value_1=1.0, Value_2=0.0, name='h01')
		h02 = mat.AddNode('math', loc=(0+x, 30+y), Math_Argument=1, Value_1=1.0, Value_2=1.0, name='h02')
		h03 = mat.AddNode('math', loc=(0+x, 60+y), Math_Argument=3, Value_1=-1.0, Value_2=1.0, name='h03')
		h04 = mat.AddNode('clouds', loc=(0+x, 90+y), Sky_Color=(0,0,0), Cloud_Color=(1,1,1), Scale=120, Complexity=12, Bottom=0, Gain=0.3, name='h04')
		h05 = mat.AddNode('math', loc=(0+x, 120+y), Math_Argument=1, Value_1=0.7, Value_2=0.7, name='h05')
		h06 = mat.AddNode('p', loc=(0+x, 150+y), x=0, y=0.8, z=0, name='h06')
		h07 = mat.AddNode('n', loc=(0+x, 180+y), x=0, y=3, z=0, name='h07')
		h08 = mat.AddNode('fractal', loc=(0+x, 210+y), x_Index=100, y_Index=100, z_Index=100, Octaves=24, Bottom=0.2, Bias=0.6, Gain=0.19, name='h08')
		
		mat.ConnectNodes(h02, h01, 'Value_1')
		mat.ConnectNodes(h03, h02, 'Value_2')
		mat.ConnectNodes(h04, h06, 'y')
		mat.ConnectNodes(h05, h02, 'Value_1')
		mat.ConnectNodes(h06, h05, 'Value_1')
		mat.ConnectNodes(h07, h05, 'Value_2')
		mat.ConnectNodes(h08, h03, 'Value_2')
		
		mat.ConnectNodes(min, h03, 'Value_1')
		
		return h01
		
	def AddSnow(self, material):
		
		mat = Mat(material)
		
		nodeDiffuse = self.PrepareDiffuse(mat)
		nodeSpecular = self.PrepareSpecular(mat)
		nodeDisplacement = self.PrepareInput(mat, 'Displacement', loc=(250, 305))
		nodeBump = self.PrepareInput(mat, 'Bump', loc=(250, 190), inp2=(0,0,0), setPS1=False)
		blend = self.ui.ctl_Smoothing.IsChecked()
		c00 = mat.AddNode('blender', loc=(250, 110), collapsed=False, Input_1=(1,1,1), Input_2=(1,1,1), Blending=blend, name='c00')
		
		self.AddSpecularNodes(mat, nodeSpecular, offset=(250, 620))
		
		# Main control nodes
		x=500; y=305
		c01 = mat.AddNode('math', loc=(0+x, 0+y), collapsed=False, Math_Argument=1, Value_1=1.0, Value_2=1.0, name="c01:Main control")
		c02 = mat.AddNode('fresnel_blend', loc=(0+x, 140+y), collapsed=False, name='c02:Snow blend')
		c03 = mat.AddNode('math', loc=(0+x, 80+y), collapsed=True, Math_Argument=3, Value_1=1, Value_2=1, name='c03')
		c04 = mat.AddNode('n', loc=(0+x, 110+y), collapsed=True, x=0.0, y=3.0, z=0.0, name='c04')
		mat.ConnectNodes(c01, c03, 'Value_1')
		mat.ConnectNodes(c01, nodeSpecular, 'Blending')
		mat.ConnectNodes(c01, nodeBump, 'Blending')
		mat.ConnectNodes(c01, nodeDiffuse, 'Blending')
		mat.ConnectNodes(c01, c00, 'Input_1')
		mat.ConnectNodes(c04, c03, 'Value_2')
		mat.ConnectNodes(c03, c00, 'Input_2')
		mat.ConnectNodes(c02, nodeDiffuse, 'Input_2')
		mat.ConnectNodes(c00, nodeDisplacement, 'Blending')
		
		# Distribution control
		x=760; y=225
		n09 = mat.AddNode('math', loc=(0+x, 0+y), collapsed=False, Math_Argument=3, Value_1=1.0, Value_2=1.0, name='n09')
		n10 = mat.AddNode('math', loc=(0+x, 80+y), collased=True, Math_Argument=2, Value_1=1.0, Value_2=1.0, name='n10')
		n11 = mat.AddNode('math', loc=(0+x, -30+y), collapsed=True, Math_Argument=3, Value_1=1.0, Value_2=1.0, name='n11')
		mat.ConnectNodes(n09, c01, 'Value_2')
		mat.ConnectNodes(n10, n09, 'Value_1')
		mat.ConnectNodes(n11, c01, 'Value_1')
		
		# Colour
		c1 = ConvertColour(self.ui.ctl_Cloud1.GetColour())
		c2 = ConvertColour(self.ui.ctl_Cloud2.GetColour())
		q = self.ui.ctl_ReflectQuality.GetValue() / 100.0
		d = self.ui.ctl_ReflectSoftness.GetValue() / 100.0
		ior = self.ui.ctl_ReflectAmount.GetValue() / 10.0

		a01 = mat.AddNode('reflect', loc=(0+x, 110+y), collapsed=True, Background=(1,1,1), Quality=q, Diffusion=d, name='a01:Reflect')
		a02 = mat.AddNode('clouds', loc=(0+x, 140+y), collapsed=True, Sky_Color=c1, Cloud_Color=c2, name='a02:Clouds')
		a03 = mat.AddNode('diffuse', loc=(0+x, 170+y), collapsed=True, Diffuse_Value=0.85, name='a03:Diffuse')
		a04 = mat.AddNode('blender', loc=(0+x, 200+y), collapsed=False, Input_1=(1,1,1), Input_2=(1,1,1), Blending=1.0, name='a04:Blender')
		
		if self.ui.ctl_SSS.IsChecked():
			a05 = mat.AddNode('scatter', loc=(0+x, 280+y), collapsed=True, Material=11, Use_Material_Color=0, Texture_Detail=1, Scatter_Group=10, name='a05')
		else:
			a05 = mat.AddNode('diffuse', loc=(0+x, 280+y), collapsed=True, Diffuse_Value=0.9, name='a05')
		
		mat.ConnectNodes(a02, a01, 'Background')
		mat.ConnectNodes(a01, a04, 'Input_2')
		mat.ConnectNodes(a03, a04, 'Input_1')
		mat.ConnectNodes(a04, c02, 'Outer_Color')
		mat.ConnectNodes(a05, c02, 'Inner_Color')
		
		# Distribution  nodes
		x=760; y=560
		n01 = mat.AddNode('math', loc=(0+x, 0+y),   collapsed=True, Math_Argument=1, Value_1=1.0, Value_2=-1.0, name='n01:Distribution')
		n02 = mat.AddNode('math', loc=(0+x, 30+y),  collapsed=True, Math_Argument=3, Value_1=1.0, Value_2=1.0, name='n02')
		n03 = mat.AddNode('math', loc=(0+x, 60+y),  collapsed=True, Math_Argument=21, Value_1=1.0, Value_2=0.001, name='n03')
		n04 = mat.AddNode('math', loc=(0+x, 90+y),  collapsed=True, Math_Argument=17, Value_1=1.0, Value_2=0.0, name='n04')
		n05 = mat.AddNode('math', loc=(0+x, 120+y), collapsed=True, Math_Argument=1, Value_1=20.0, Value_2=-2.0, name='n05')
		n06 = mat.AddNode('math', loc=(0+x, 150+y), collapsed=True, Math_Argument=1, Value_1=1.3, Value_2=0.1, name='n06')
		n07 = mat.AddNode('n', loc=(0+x, 180+y), collapsed=True, x=0.0, y=1.0, z=0.0, name='n07')
		n08 = mat.AddNode('turbulence', loc=(0+x, 210+y), collapsed=True, x_Index=1.0, y_Index=1.0, z_Index=1.0, Octaves=2.0, Noise_Type=2.0, name='n08')

		mat.ConnectNodes(n01, n09, 'Value_2')
		mat.ConnectNodes(n02, n01, 'Value_2')
		mat.ConnectNodes(n03, n02, 'Value_2')
		mat.ConnectNodes(n04, n03, 'Value_1')
		mat.ConnectNodes(n05, n04, 'Value_1')
		mat.ConnectNodes(n06, n05, 'Value_1')
		mat.ConnectNodes(n07, n06, 'Value_1')
		mat.ConnectNodes(n08, n06, 'Value_2')
		
		# Parameters
		x=1020; y=20
			
		# Get data from UI
		pAmount = self.ui.ctl_Amount.GetValue() / 100.0
		pBump = self.ui.ctl_Bump.GetValue() / 100.0
		r, g, b = self.ui.ctl_Color.GetColour()
		pColour = (r/255.0, g/255.0, b/255.0)
		pDepth = self.ui.ctl_Depth.GetValue() / 100.0
		pWiggle = self.ui.ctl_Wiggle.GetValue()
		pMinHeight = self.ui.ctl_Height.GetValue()
		pUseHeight = self.ui.ctl_LimitByHeight.IsChecked()
		pReflect = self.ui.ctl_Reflect.IsChecked()
		pTopOnly = self.ui.ctl_TopOnly.IsChecked()
		
		p01 = mat.AddNode('math', loc=(0+x, 0+y), collapsed=False, Math_Argument=1, Value_1=pBump, name='BUMP')
		p02 = mat.AddNode('math', loc=(0+x, 80+y), collapsed=False, Math_Argument=3, Value_1=pDepth, Value_2=1.0, name='DEPTH')
		p03 = mat.AddNode('clouds', loc=(0+x, 160+y), collapsed=True, Sky_Color=(0,0,0), Cloud_Color=(1,1,1), Scale=2.0, name='p03')
		mat.ConnectNodes(p03, p02, 'Value_2')
		p04 = mat.AddNode('math', loc=(0+x, 220+y), collapsed=False, Math_Argument=1, Value_1=pUseHeight, name='USE_HEIGHT')
		mat.ConnectNodes(p04, n11, 'Value_1')
		mat.ConnectNodes(p04, n10, 'Value_2')
		p05 = mat.AddNode('math', loc=(0+x, 300+y), collapsed=False, Math_Argument=1, Value_1=pMinHeight, name='MIN_HEIGHT')
		p06 = mat.AddNode('math', loc=(0+x, 380+y), collapsed=False, Math_Argument=1, Value_1=pReflect, name='REFLECT')
		mat.ConnectNodes(p06, a04, 'Blending')
		p07 = mat.AddNode('color', loc=(0+x, 460+y), collapsed=False, Color=pColour, name='SNOW_COLOR')
		mat.ConnectNodes(p07, a03, 'Diffuse_Color')
		if self.ui.ctl_SSS.IsChecked():
			mat.ConnectNodes(p07, a05, 'Color')
		else:
			mat.ConnectNodes(p07, a05, 'Diffuse_Color')
		p08 = mat.AddNode('math', loc=(0+x, 540+y), collapsed=False, Math_Argument=1, Value_1=pTopOnly, name='TOP_ONLY')
		mat.ConnectNodes(p08, n02, 'Value_1')
		#mat.ConnectNodes(p08, c00, 'Blending')
		p09 = mat.AddNode('math', loc=(0+x, 620+y), collapsed=False, Math_Argument=1, Value_1=pAmount, name='AMOUNT')
		mat.ConnectNodes(p09, n07, 'y')
		p10 = mat.AddNode('math', loc=(0+x, 700+y), collapsed=False, Math_Argument=1, Value_1=pWiggle, name='WIGGLE')
		mat.ConnectNodes(p10, n08, 'x_Index')
		mat.ConnectNodes(p10, n08, 'y_Index')
		mat.ConnectNodes(p10, n08, 'z_Index')
		
		# Height controls
		if self.ui.ctl_LimitByHeight.IsChecked():
			h = self.AddHeightNodes(mat, p05, offset=(500, 560))
			mat.ConnectNodes(h, n11, 'Value_2')
			
		# Bump and displacement nodes
		b = self.AddBumpNodes(mat, p01, offset=(500, 20))
		d = self.AddDisplacementNodes(mat, p02, offset=(760, 20))
		
		bd01 = mat.AddNode('math', loc=(500, 205), collapsed=False, Math_Argument=1.0, Value_1=1.0, Value_2=1.0, name='Lumpiness')
		mat.ConnectNodes(b, bd01, 'Value_1')
		mat.ConnectNodes(d, bd01, 'Value_2')
		mat.ConnectNodes(bd01, nodeDisplacement, 'Input_2')
		
	def AddSpecularNodes(self, mat, blender, offset=None):

		(NIX, ADD, SUBTRACT, MULTIPLY, DIVIDE, SINE, COSINE, TANGENT, SQRT, POWER, EXPONENTIAL, LOGARITHM, MODULUS, ABSOLUTE, SIGN, MINIMUM, MAXIMUM, CLAMP, CEILING, FLOOR, ROUND, STEP, SMOOTHSTEP, BIAS, GAIN) = range(25)
	
		if offset:
			x, y = offset
		else:
			x = y = 0
		
		sc = ConvertColour(self.ui.ctl_SpecColor.GetColour())
		e = self.ui.ctl_Eccentricity.GetValue() / 100.0
		ro = self.ui.ctl_RollOff.GetValue() / 100.0
		r = self.ui.ctl_Reflectivity.GetValue() / 100.0
		
		s01 = mat.AddNode('blinn', name='s01:SSMBlinn', loc=(0+x, 0+y), collapsed=True, Specular_Color=sc, Eccentricity=e, SpecularRollOff=ro, Reflectivity=r)
		mat.ConnectNodes(s01, blender, 'Input_2')
		s02 = mat.AddNode('math', loc=(0+x, 30+y), Math_Argument=GAIN, Value_1=1.0, Value_2=0.8, name='s02')
		mat.ConnectNodes(s02, s01, 'Reflectivity')
		s03 = mat.AddNode('math', loc=(0+x, 60+y), Math_Argument=CLAMP, Value_1=1.0, Value_2=0.0, name='s03')
		mat.ConnectNodes(s03, s02, 'Value_1')
		s04 = mat.AddNode('noise', loc=(0+x, 90+y), min=-2.0, name='s04')
		mat.ConnectNodes(s04, s03, 'Value_1')
	
	def GetActorData(self, actor):
		if actor:
			if self.ValidActor(actor, bodypart=True):
				figure = actor.ItsFigure()
				if figure:
					name = figure.Name()
					type = "Figure"
					self.actor = self.actor.ItsFigure()
				else:
					name = actor.Name()
					type = "Prop"
			else:
				name = actor.Name()
				type = 'Invalid actor'
		else:
			name = "None"
			type = "No actor"
		
		return name, type
		
	def GetActorName(self):
		return self.name
	
	def GetActorType(self):
		return self.type

	def IsHidden(self, actor):
		hidden = False
		if actor.OnOff() == 0:
			hidden = True
		else:
			p = actor.Parent()
			if p:
				if p.Name() != 'UNIVERSE':
					if p.IsBodyPart():
						hidden = self.IsHiddenFig(p.ItsFigure())
					elif p.InternalName().startswith('Grouping') and p.OnOff() == 0:
						hidden = True
					else:
						hidden = self.IsHidden(p)
		return hidden
		
	def IsHiddenFig(self, actor):
		hidden = False
		if actor.Visible() == 0:
			hidden = True
		else:
			parent = actor.ParentActor()
			if parent:
				if parent.Name() != 'UNIVERSE':
					p = parent.Parent()
					if p:
						if p.InternalName().startswith('Grouping') and p.OnOff() == 0:
							hidden = True
		return hidden
		
	def MakeSnow(self):
		
		if self.ui.ctl_AllMaterials.IsChecked():
			done = []
			scene = poser.Scene()
			
			if self.ui.ApplyToAllProps() == True:
				for actor in scene.Actors():
					if self.type == 'Prop' and (actor.Name() == self.name): continue
					if self.ValidActor(actor) and not self.IsHidden(actor):
						self.Mats(actor)
						done.append(actor)
			
			if self.ui.ApplyToAllFigures() == True:
				for fig in scene.Figures():
					if self.type == 'Figure' and (fig.Name() == self.name): continue
					if not self.IsHiddenFig(fig):
						self.Mats(fig)
						done.append(fig)
		
			if not self.type in ('Invalid actor', 'No actor'):
				if self.type == 'Figure':
					if not self.IsHiddenFig(self.actor):
						self.Mats(self.actor)
				else:
					if not self.IsHidden(self.actor):
						self.Mats(self.actor)
				
		else:
			mats = poser.Scene().WacroMaterials()
			if (mats == None):
				ShowAlert("No Materials found")
			else:
				self.Mats(None, mats)

		ShowAlert('All done!', 'The Snarlygribbly Snow Machine')
		
	def Mats(self, actor, mats=None):

		if mats == None: mats = actor.Materials()
		
		if actor:
			try:
				if actor.IsProp() and actor.IsBodyPart():
					try:
						name = actor.ItsFigure().Name()
					except:
						name = actor.Name()
				else:
					name = actor.Name()
			except:
				name = actor.Name()
		else:
			name = 'Current actor'
		
		if mats:		
			for m in mats:
				self.ui.ctl_StatusBar.SetStatusText("Processing material '%s' of actor '%s'" % (m.Name(), name))
				self.AddSnow(m)
				removeOrphans(m)
	
	def PrepareDiffuse(self, mat):
	
		blend = self.ui.ctl_Opacity.GetValue() / 100.0
		
		inNode = mat.GetPSNode('AlternateDiffuse')
		
		if inNode:
			if inNode.Name().startswith('SSM'):
				inNode=inNode.Input(0).InNode()
		
		if inNode:
			colour = mat.GetPSValue('AlternateDiffuse')
			blender = mat.AddNode('blender', name='SSMTheSnow', loc=(250, 385), collapsed=False, Input_1=colour, Blending=blend)
			mat.ConnectToPS(blender, 'AlternateDiffuse', value=(1.0, 1.0, 1.0))
			mat.ConnectNodes(inNode, blender, 'Input_1')
		else:
			colour = mat.GetPSValue('Diffuse_Color')
			value = mat.GetPSValue('Diffuse_Value')
			inNode = mat.GetPSNode('Diffuse_Color')
			diffuse = mat.AddNode('diffuse', loc=(250, 465), Diffuse_Color=colour, Diffuse_Value=value)
			if inNode: mat.ConnectNodes(inNode, diffuse, 'Diffuse_Color')
			blender = mat.AddNode('blender', name='SSMTheSnow', loc=(250, 385), collapsed=False, Blending=blend)
			mat.ConnectToPS(blender, 'AlternateDiffuse', value=(1.0, 1.0, 1.0))
			mat.ConnectNodes(diffuse, blender, 'Input_1')

			mat.SetPSValue('Diffuse_Value', 0.0)
			mat.RemovePSNode('Diffuse_Color')
			
		return blender
		
	def PrepareInput(self, mat, input, loc, inp2=(1,1,1), setPS1=True):
		inp = mat.root.InputByInternalName(input)
		
		oldnode = inp.InNode()

		if oldnode:
			if oldnode.Name().startswith('SSM'):
				oldnode = oldnode.Input(0).InNode()
		
		if oldnode:
			inp1 = (1,1,1)
		else:
			inp1 = (0,0,0)
		
		v = mat.GetPSValue(input)
		if setPS1 == True and v == 0: v = 1
			
		blender = mat.AddNode('blender', name='SSM'+input, loc=loc, collapsed=False, Input_1=inp1, Input_2=inp2, Blending=1.0)
		mat.ConnectToPS(blender, input, value=v)
		if oldnode:
			mat.ConnectNodes(oldnode, blender, 'Input_1')
			
		return blender

	def PrepareSpecular(self, mat):
	
		blend = self.ui.ctl_Opacity.GetValue() / 100.0
		
		inNode = mat.GetPSNode('AlternateSpecular')
		
		if inNode:
			if inNode.Name().startswith('SSM'):
				inNode = inNode.Input(0).InNode()
		
		if inNode:
			colour = mat.GetPSValue('AlternateSpecular')
			blender = mat.AddNode('blender', name='SSMSpecular', loc=(250, 510), collapsed=False, Input_1=colour, Blending=blend)
			mat.ConnectToPS(blender, 'AlternateSpecular', value=(1.0, 1.0, 1.0))
			mat.ConnectNodes(inNode, blender, 'Input_1')
		else:
			colour = mat.GetPSValue('Highlight_Color')
			value = mat.GetPSValue('Highlight_Value')
			roughness = mat.GetPSValue('Roughness')
			specular = mat.AddNode('specular', loc=(250, 590), Specular_Color=colour, Specular_Value=value, Roughness=roughness)
			
			inNode = mat.GetPSNode('Highlight_Color')
			if inNode: mat.ConnectNodes(inNode, specular, 'Specular_Color')
			inNode = mat.GetPSNode('Highlight_Value')
			if inNode: mat.ConnectNodes(inNode, specular, 'Specular_Value')
			inNode = mat.GetPSNode('Roughness')
			if inNode: mat.ConnectNodes(inNode, specular, 'Roughness')
			
			blender = mat.AddNode('blender', name='SSMSpecular', loc=(250, 510), collapsed=False, Blending=blend)
			mat.ConnectToPS(blender, 'AlternateSpecular', value=(1.0, 1.0, 1.0))
			mat.ConnectNodes(specular, blender, 'Input_1')

			mat.SetPSValue('Highlight_Value', 0.0)
			mat.RemovePSNode('Highlight_Color')
			mat.RemovePSNode('Highlight_Value')
			mat.RemovePSNode('Roughness')
			
		return blender		
	
	def ValidActor(self, actor, bodypart=False):
		n = actor.Name()
		if n in ('UNIVERSE', 'FocusDistanceControl') or n.startswith('GoalCenterOfMass') or n.startswith('CenterOfMass') or n.startswith('Grouping'):
			return False
		
		if actor.IsProp() or (actor.IsBodyPart() and bodypart == True):
			return True
		else:
			return False
		
class UI(mainFrame):
	def __init__(self):
		mainFrame.__init__(self, None)
		
		self.sm = SnowMachine(self)
		
		self.txt_CurrentActor.SetLabel(self.sm.GetActorName())
		self.txt_ActorType.SetLabel(self.sm.GetActorType())
		
		poser.Scene().SetEventCallback(self.ActorChanged)
		
		self.controls = {
			'slider_amount' : (self.ctl_Amount, 'int', 'std'),
			'text_amount' : (self.txt_Amount, 'str', 'std'),
			'slider_depth' : (self.ctl_Depth, 'int', 'std'),
			'text_depth' : (self.txt_Depth, 'str', 'std'),
			'slider_wiggle' : (self.ctl_Wiggle, 'int', 'std'),
			'text_wiggle' : (self.txt_Wiggle, 'str', 'std'),
			'slider_bump' : (self.ctl_Bump, 'int', 'std'),
			'text_bump' : (self.txt_Bump, 'str', 'std'),
			'reflect' : (self.ctl_Reflect, 'bool', 'std'),
			'smooth' : (self.ctl_Smoothing, 'bool', 'std'),
			'by_height' : (self.ctl_LimitByHeight, 'bool', 'std'),
			'minheight' : (self.ctl_Height, 'int', 'std'),
			'txtheight' : (self.txt_Height, 'str', 'std'),
			'color' : (self.ctl_Color, 'rgb', 'col'),
			'spec_col' : (self.ctl_SpecColor, 'rgb', 'col'),
			'eccentricity' : (self.ctl_Eccentricity, 'int', 'std'),
			'rolloff' : (self.ctl_RollOff, 'int', 'std'),
			'reflectivity' : (self.ctl_Reflectivity, 'int', 'std'),
			'quality' : (self.ctl_ReflectQuality, 'int', 'std'),
			'softness' : (self.ctl_ReflectSoftness, 'int', 'std'),
			'colour1' : (self.ctl_Cloud1, 'rgb', 'col'),
			'colour2' : (self.ctl_Cloud2, 'rgb', 'col'),
			'reflect_amount' : (self.ctl_ReflectAmount, 'int', 'std'),
			'sss' : (self.ctl_SSS, 'bool', 'std'),
			'opacity' : (self.ctl_Opacity, 'int', 'std')
		}
		
		# Read the presets configuraton file
		self.config = ConfigParser.RawConfigParser()
		self.presetsfile = os.path.join(poser.PrefsLocation(), 'snowmachine' + os.extsep + 'cfg')
		self.config.read(self.presetsfile)
		self.modified = False
		
		# Initialise the Presets menu
		self.InitialisePresets()
	
	def __del__( self ):
		poser.Scene().ClearEventCallback()
		# Save our presets
		if self.modified == True:
			with open(self.presetsfile, 'wb') as cfgfile:
				self.config.write(cfgfile)
		
	def ActorChanged(self, scene, event):
		if (event & poser.kEventCodeACTORSELECTIONCHANGED):
				self.sm.ActorData()
				self.txt_CurrentActor.SetLabel(self.sm.GetActorName())
				self.txt_ActorType.SetLabel(self.sm.GetActorType())
		self.SetFocus()

	def AddPresetToMenu(self, name):
		
		# Create a menu item
		item = wx.MenuItem( self.mnu_Presets, wx.ID_ANY, name, wx.EmptyString, wx.ITEM_NORMAL )
		
		# Add it to the menu
		self.mnu_Presets.AppendItem(item)
		
		# Link it to our self.ApplyPreset method
		self.Bind( wx.EVT_MENU, self.ApplyPreset, id = item.GetId() )	
		
	def AllMats( self, event ):
		if self.ctl_AllMaterials.IsChecked():
			self.ctl_AddSnow.Enable(True)
		else:
			self.ctl_AddSnow.Enable(False)

	def ApplyPreset(self, event):
		
		# Find the relevant menu item
		id = event.GetId()
		item = self.mnu_Presets.FindItemById(id)
		
		# Get its name
		preset = item.GetLabel()
		
		if self.config.has_section(preset):	# Check that the preset exists in the configuration file
		
			# Get all the settings for that preset
			for option, value in self.config.items(preset):
				
				# Make sure our dictionary knows about the setting
				if self.controls.has_key(option):
					
					# Get a reference to our control and its type by looking it up in our dictionary
					control, type, nix = self.controls[option]
					
					# Read and apply the value
					if type == 'int':
						control.SetValue(self.config.getint(preset, option))
					if type == 'float':
						control.SetValue(self.config.getfloat(preset, option))
					elif type == 'bool':
						control.SetValue(self.config.getboolean(preset, option))
					elif type == 'str':
						control.SetValue(self.config.get(preset, option))
					elif type == 'rgb':
						control.SetColour(self.config.get(preset, option))

				else:
					ShowAlert("Unknown option '%s' encountered" % option) 
		else:
			ShowAlert('Preset %s not found in presets file' % preset)
		
	def ApplyToAllMaterials(self):
		return self.ctl_AllMaterials.IsChecked()

	def ApplyToAllFigures(self):
		if self.ctl_AddSnow.GetSelection() == 2:
			return True
		else:
			return False
			
	def ApplyToAllProps(self):
		if self.ctl_AddSnow.GetSelection() > 0:
			return True
		else:
			return False
			
	def ApplyToCurrentOnly(self):
		if self.ctl_AddSnow.GetSelection() == 0:
			return True
		else:
			return False	

	def DeletePreset(self, name):
		self.config.remove_section(name)
		id = self.mnu_Presets.FindItem(name)
		if id:
			self.mnu_Presets.DeleteItem(self.mnu_Presets.FindItemById(id))
		self.modified = True
		
	def GetFromScene( self, event ):
		
		parms = (
			('BUMP', 'int', self.ctl_Bump, self.txt_Bump, 'Value_1', 100),
			('DEPTH', 'int', self.ctl_Depth, self.txt_Depth, 'Value_1', 100),
			('USE_HEIGHT', 'bool', self.ctl_LimitByHeight, None, 'Value_1', False),
			('MIN_HEIGHT', 'int', self.ctl_Height, self.txt_Height, 'Value_1', False),
			('REFLECT', 'bool', self.ctl_Reflect, None, 'Value_1', False),
			('SNOW_COLOR', 'color', self.ctl_Color, None, 'Color', False),
			('TOP_ONLY', 'bool', self.ctl_TopOnly, None, 'Value_1', False),
			('AMOUNT', 'int', self.ctl_Amount, self.txt_Amount, 'Value_1', 100),
			('WIGGLE', 'int', self.ctl_Wiggle, self.txt_Wiggle, 'Value_1', False),
			('c00', 'bool', self.ctl_Smoothing, None, 'Blending', False),
			('s01:SSMBlinn', 'color', self.ctl_SpecColor, None, 'Specular_Color', False),
			('s01:SSMBlinn', 'int', self.ctl_Eccentricity, None, 'Eccentricity', 100),
			('s01:SSMBlinn', 'int', self.ctl_RollOff, None, 'SpecularRollOff', 100),
			('s01:SSMBlinn', 'int', self.ctl_Reflectivity, None, 'Reflectivity', 100),
			('a01:Reflect', 'int', self.ctl_ReflectQuality, None, 'Quality', 100),
			('a01:Reflect', 'int', self.ctl_ReflectSoftness, None, 'Diffusion', 100),
			('c02:Snow blend', 'int', self.ctl_ReflectAmount, None, 'Index_of_Refraction', 10),
			('a02:Clouds', 'color', self.ctl_Cloud1, None, 'Sky_Color', False),
			('a02:Clouds', 'color', self.ctl_Cloud2, None, 'Cloud_Color', False)
			)
		
		currentMAT = poser.Scene().CurrentMaterial()
		if currentMAT:
			d = {}
			st = currentMAT.ShaderTree()
			for n in st.Nodes():
				d[n.Name()] = n.InternalName()
			
			noneFound = True
			
			for value in parms:
				inp = self.GetInput(value, d, st)
				if inp:
					noneFound = False
					if value[1] == 'int':
						v = round(inp.Value(),2)
						if value[3]: value[3].SetValue(str(v))
						if value[5]: v *= value[5]
						value[2].SetValue(v)
					elif value[1] == 'bool':
						v = inp.Value()
						value[2].SetValue(v)
					elif value[1] == 'color':
						v = inp.Value()
						r, g, b = v
						value[2].SetColour((r*255.0, g*255.0, b*255.0))
					else:
						continue
						
			node = d.get('a05', None)
			if node:
				node = st.NodeByInternalName(node)
				self.ctl_SSS.SetValue((True, False)[node.Type() == poser.kNodeTypeCodeDIFFUSE])
			
			if noneFound:
				ShowAlert('No valid material selected')
			else:
				ShowAlert('Data taken from %s material' % currentMAT.Name(), "Snarly's Snow Machine")
			
		else:
			ShowAlert('No valid material selected')
	
	def GetInput(self, value, d, st):
		name = d.get(value[0], None)
		if name:
			node = st.NodeByInternalName(name)
			if node:
				inp = node.InputByInternalName(value[4])

				if inp:
					return inp
				else:
					return None
	
	def Icy( self, event ):
		self.ctl_Eccentricity.SetValue(8)
		self.ctl_RollOff.SetValue(40)
		self.ctl_Reflectivity.SetValue(700)

	def InitialisePresets(self):
		# get a list of all presets from our configuration file
		presets = self.config.sections()
		
		# And add them to the menu
		for preset in presets:
			self.AddPresetToMenu(preset)
			
	def MakeSnow( self, event ):
		self.ctl_StatusBar.SetStatusText('Making snow ...')
		self.sm.MakeSnow()
		self.ctl_StatusBar.SetStatusText('')

	def ManagePreset( self, event ):
	
		poser.Scene().ClearEventCallback()
		
		# Create the dialog and show it
		dlg = SelectPresets(self.config.sections(), 'Delete selected')
		retcode = dlg.ShowModal()
		
		# Process the user's selection
		if retcode == 1:
			for preset in dlg.ctl_PresetList.GetCheckedStrings():
				self.DeletePreset(preset)
		
		# Delete the dialog
		dlg.Destroy()
		
		poser.Scene().SetEventCallback(self.ActorChanged)

	def MenuSave( self, event ):
	
		# Create a dictionary to hold all the settings
		settings = {}
		for key, value in self.controls.items():
			if value[2] == "std":
				settings[key] = str(value[0].GetValue())
			elif value[2] == "sel":
				settings[key] = str(value[0].GetSelection())
			else:
				settings[key] = str(value[0].GetColour())
		
		# Get the root element
		root = ET.Element('SSM')
		tree = ET.ElementTree(root)
		
		# Add the settings to the XML
		element = ET.Element('Settings', settings)
		root.append(element)	
		
		# Choose a file to save to
		dlg = wx.FileDialog(self, 'Save file', '', '', '*.SSM2', wx.FD_SAVE|wx.FD_OVERWRITE_PROMPT)
		if dlg.ShowModal() == wx.ID_OK:
			path = dlg.GetPath()
			file = open(path, 'w')

			# Save to disk
			if file:
				tree.write(file)
				try: file.close()
				except: pass
			else:
				ShowAlert('Unable to save file')
		dlg.Destroy()
				
	def MenuLoad( self, event ):
		
		# Get file
		dlg = wx.FileDialog(self, 'Open file', '', '', '*.SSM2', wx.FD_OPEN)
		if dlg.ShowModal() == wx.ID_OK:
			path = dlg.GetPath()
			file = open(path, 'r')
			if file:
				
				# Make sure we have a valid XML file
				try:
					tree = ET.parse(file)
					root = tree.getroot()
				except IOError, err:
					ShowAlert("Cannot open file"); return
				except Exception, err:
					ShowAlert("The Snow Machine encountered a problem with:\n%s\n%s" % (path, err)); return
					
				# Read values
				element = root.find('Settings')
				for key, value in element.items():
					control, datatype, type  = self.controls[key]
					
					if datatype == 'str':
						data = value
					elif datatype == 'bool':
						data = (True, False)[value == 'False']
					elif datatype == 'int':
						data = int(value)
					elif datatype == 'rgb':
						data = eval(value)
					else:
						data = float(value)
						
					if type == 'std':
						control.SetValue(data)
					elif type == 'col':
						control.SetColour(data)
					else:
						control.SetSelection(data)
			
			else:
				ShowAlert('Unable to open file')
	
	def PresetExport( self, event ):
	
		poser.Scene().ClearEventCallback()

		# Create a dialog box to select presets for export
		dlg = SelectPresets(self.config.sections(), buttontext='Export')
		retcode = dlg.ShowModal()
		
		# Process the selection
		if retcode == 1:						# User pressed the 'Export' button
		
			presets = dlg.ctl_PresetList.GetCheckedStrings()
			if len(presets) > 0:				# Make sure something was selected
			
				# Get a config parser ...
				export = ConfigParser.RawConfigParser()
				
				# ... and copy selected presets into it
				for p in presets:
					export.add_section(p)
					for opt, val in self.config.items(p):
						export.set(p, opt, val)
				
				# Ask the user for a save location
				fdlg = wx.FileDialog(None,
					'Choose a file',
					defaultDir='',
					defaultFile='snowmachine_export.cfg',
					wildcard='*.cfg',
					style=wx.SAVE,
					pos=wx.DefaultPosition)
				
				# Save the file
				if fdlg.ShowModal() == wx.ID_OK:
					path = fdlg.GetPath()
					with open(path, 'wb') as cfgfile:
						export.write(cfgfile)
					ShowAlert('Presets successfully exported','Preset exporter')
					fdlg.Destroy()
					
		# Restore the UI
		self.SetFocus()
		
		poser.Scene().SetEventCallback(self.ActorChanged)
	
	def PresetImport( self, event ):
	
		poser.Scene().ClearEventCallback()
		
		# Ask the user for a file location
		fdlg = wx.FileDialog(None,
			'Choose a file',
			defaultDir='',
			defaultFile='',
			wildcard='*.cfg',
			style=wx.OPEN,
			pos=wx.DefaultPosition)
			
		if fdlg.ShowModal() == wx.ID_OK:			# User chose a file
		
			path = fdlg.GetPath()					# get the path to the file
		
			# Get a config parser
			imp = ConfigParser.RawConfigParser()
			
			# And read the config data from the user's chosen file
			imp.read(path)
			
			presets = imp.sections()
			
			# Create a dialog box to select presets for import
			dlg = SelectPresets(presets, buttontext='Import')
			retcode = dlg.ShowModal()
			
			presets = dlg.ctl_PresetList.GetCheckedStrings()
			
			# Get each preset in turn
			for preset in presets:
			
				# We need to make sure preset names are unique
				id = 0
				newPreset = preset
				
				# Rename if necessary ...
				while self.config.has_section(newPreset):
					newPreset = '%s_%s' % (preset, id)
					id += 1
				
				# Add the imported preset to our presets
				self.config.add_section(newPreset)
				for opt, val in imp.items(preset):
					self.config.set(newPreset, opt, val)
				
				# Update the script's menu
				self.AddPresetToMenu(newPreset)
				
				self.modified = True
			
			# Advise the user of completion of import
			if len(presets) > 0: ShowAlert('Import successful', 'Preset importer')
			
		#Restore the UI window
		self.SetFocus()
		
		poser.Scene().SetEventCallback(self.ActorChanged)
				
	def Quit( self, event ):
		self.Close()

	def SavePreset( self, event ):
	
		poser.Scene().ClearEventCallback()
		
		# Ask the user to give the preset aname
		dialog = wx.TextEntryDialog(None, "Preset name:", "Text Entry", "Default Value", style=wx.OK|wx.CANCEL)
		
		if dialog.ShowModal() == wx.ID_OK:	# And if they do ...
		
			# Make a note of the name
			preset = dialog.GetValue()
			
			# The name 'DEFAULT' is reserved for our use
			if preset.lower() == 'default':
				ShowAlert("You are not allowed to use the name '%s'" % preset)
			
			# If the preset already exists, ask the user whether or not it should be overwritten
			elif self.config.has_section(preset):
			
				# Contruct the dialog box
				dlg = wx.MessageDialog(None, "Overwrite existing preset?", "Preset name already exists", wx.YES_NO | wx.ICON_QUESTION)
				
				# Check that the user said it was ok to overwrite the preset
				if dlg.ShowModal() == wx.ID_YES:
					
					# Destroy the dialog box and update the preset settings
					dlg.Destroy()
					self.UpdatePreset(preset)
			else:
				
				# Create the preset in the configuration file
				self.config.add_section(dialog.GetValue())
				
				# And add it to the menu
				self.AddPresetToMenu(preset)
				
				# And update the settings for the preset in the configuration file
				self.UpdatePreset(preset)
				
		dialog.Destroy()
		
		poser.Scene().SetEventCallback(self.ActorChanged)
		
	def Snowy( self, event ):
		self.ctl_Eccentricity.SetValue(70)
		self.ctl_RollOff.SetValue(30)
		self.ctl_Reflectivity.SetValue(100)
	
	def SetDefaults( self, event ):
		self.ctl_Amount.SetValue(50); self.txt_Amount.SetValue('0.50')
		self.ctl_Depth.SetValue(30); self.txt_Depth.SetValue('0.30')
		self.ctl_Wiggle.SetValue(3); self.txt_Wiggle.SetValue('3')
		self.ctl_Bump.SetValue(2); self.txt_Bump.SetValue('0.02')
		self.ctl_Color.SetColour('WHITE')
		self.ctl_Reflect.SetValue(wx.CHK_CHECKED)
		self.ctl_LimitByHeight.SetValue(wx.CHK_UNCHECKED)
		self.ctl_Height.SetValue(100); self.txt_Height.SetValue('100')
		self.ctl_Smoothing.SetValue(wx.CHK_CHECKED)
		self.ctl_TopOnly.SetValue(wx.CHK_CHECKED)
		self.ctl_SpecColor.SetColour('WHITE')
		self.ctl_Eccentricity.SetValue(70)
		self.ctl_RollOff.SetValue(30)
		self.ctl_Reflectivity.SetValue(100)
		self.ctl_ReflectQuality.SetValue(20)
		self.ctl_ReflectSoftness.SetValue(0)
		self.ctl_Cloud1.SetColour('#ccccff')
		self.ctl_Cloud2.SetColour('WHITE')
		self.ctl_ReflectAmount.SetValue(14)
		self.ctl_SSS.SetValue(wx.CHK_CHECKED)

	def TopOnlyChanged( self, event ):
		self.ctl_Smoothing.SetValue(self.ctl_TopOnly.GetValue())
		
	def UpdateAmount( self, event ):
		self.UpdateTextSlider(self.txt_Amount, self.ctl_Amount, True, event, useSlider=(False, True)[event.GetId() == 1001])

	def UpdateDepth( self, event ):
		self.UpdateTextSlider(self.txt_Depth, self.ctl_Depth, True, event, useSlider=(False, True)[event.GetId() == 1003])

	def UpdateWiggle( self, event ):
		self.UpdateTextSlider(self.txt_Wiggle, self.ctl_Wiggle, False, event, useSlider=(False, True)[event.GetId() == 1005])

	def UpdateReflect( self, event ):
		self.UpdateTextSlider(self.txt_Reflect, self.ctl_Reflect, True, event, useSlider=(False, True)[event.GetId() == 1007])
	
	def UpdateBump( self, event ):
		self.UpdateTextSlider(self.txt_Bump, self.ctl_Bump, True, event, useSlider=(False, True)[event.GetId() == 1009])

	def UpdateHeight( self, event ):
		self.UpdateTextSlider(self.txt_Height, self.ctl_Height, False, event, useSlider=(False, True)[event.GetId() == 1011])

	def UpdatePreset(self, preset):
		
		# If the preset doesn't exist yet, then create it
		if not self.config.has_section(preset):
			self.config.add_section(preset)
		
		# Add or update config options to preset
		for option, value in self.controls.items():
			control, type, nix = value
			if type == 'str':
				self.config.set(preset, option, control.GetValue())
			elif type in ('int', 'float', 'bool'):
				self.config.set(preset, option, str(control.GetValue()))
			elif type == 'rgb':
				self.config.set(preset, option, control.GetColour().GetAsString())
			elif type == 'tex':
				selected = control.GetFirstSelected()
				if selected >= 0:
					self.config.set(preset, option, control.GetItemText(selected))
		
		self.modified = True
		
	def UpdateTextSlider(self, textCtl, sliderCtl, makeFloat, event, useSlider=True):
		
		if useSlider:
			if makeFloat:
				textCtl.SetValue('%.2f' % (sliderCtl.GetValue() / 100.0))
			else:
				textCtl.SetValue('%s' % sliderCtl.GetValue())
				
		else:
		
			factor = (1, 100.0)[makeFloat]
			
			min = sliderCtl.GetMin()
			max = sliderCtl.GetMax()
			value = GetFloatFromTextCtl(textCtl, factor)
			
			if value is not None:
				
				if IsBetween(min, max, value):
					sliderCtl.SetValue(value)
				else:
					if value > max:
						sliderCtl.SetValue(max)
						self.UpdateTextSlider(textCtl, sliderCtl, makeFloat, event, useSlider=True)
					else:
						sliderCtl.SetValue(min)
						if event.GetEventType() == wx.wxEVT_COMMAND_TEXT_ENTER:
							textCtl.SetValue('%.2f' % (min / factor))
					
			else:
				textCtl.SetValue('%.2f' % (sliderCtl.GetValue() / factor))	

class SelectPresets(PresetEditor):
	def __init__(self, names, buttontext=None):
		PresetEditor.__init__(self, None)
		
		if buttontext:
			self.ctl_SelectPresets.SetLabel(buttontext)
		
		for name in names:
			self.ctl_PresetList.Append(name)
			
	def __del__( self ):
		pass

	def SelectAll( self, event ):
		self.ctl_PresetList.SetChecked(range(self.ctl_PresetList.GetCount()))
	
	def SelectNone( self, event ):
		self.ctl_PresetList.SetChecked(())
		
	def SelectPresets( self, event ):
		self.EndModal(1)
	
	def Cancel( self, event ):
		self.EndModal(0)	

app = Main()		
		

